@@ -10,6 +10,7 @@
 
 #include <algorithm>
 #include <iostream>
+#include <fstream>
 #include <memory>
 #include <string>
 #include <utility>
@@ -25,7 +26,7 @@
 #include "xeus-python/xinterpreter.hpp"
 #include "xeus-python/xeus_python_config.hpp"
 
-#include "xcomm.hpp"
+#include "xpython_kernel.hpp"
 #include "xdisplay.hpp"
 #include "xinput.hpp"
 #include "xinspect.hpp"
@@ -76,9 +77,7 @@ namespace xpyt
         py::module sys = py::module::import("sys");
 
         // Monkey patching "import linecache". This monkey patch does not work with Python2.
-#if PY_MAJOR_VERSION >= 3
         sys.attr("modules")["linecache"] = get_linecache_module();
-#endif
 
         py::module jedi = py::module::import("jedi");
         jedi.attr("api").attr("environment").attr("get_default_environment") = py::cpp_function([jedi] () {
@@ -96,12 +95,18 @@ namespace xpyt
 
         // Add get_ipython to global namespace
         py::globals()["get_ipython"] = get_kernel_module().attr("get_ipython");
-        
+
         // Initializes get_ipython result
         get_kernel_module().attr("get_ipython")();
 
         m_has_ipython = get_kernel_module().attr("has_ipython").cast<bool>();
-        //m_has_ipython = true;
+
+        // Initialize cached inputs
+        py::globals()["_i"] = "";
+        py::globals()["_ii"] = "";
+        py::globals()["_iii"] = "";
+
+        load_extensions();
     }
 
     nl::json interpreter::execute_request_impl(int execution_count,
@@ -129,26 +134,26 @@ namespace xpyt
             // - either we reimplement the parsing logic in xeus-python
             // - or this logic is extracted from IPython into a dedicated package, that becomes
             // a dependency of both xeus-python and IPython.
-            if (code.size() >= 2 && code[0] == '?')	
-            {	
-                std::string result = formatted_docstring(code);	
-                if (result.empty())	
-                {	
-                    result = "Object " + code.substr(1) + " not found.";	
-                }	
+            if (code.size() >= 2 && code[0] == '?')
+            {
+                std::string result = formatted_docstring(code);
+                if (result.empty())
+                {
+                    result = "Object " + code.substr(1) + " not found.";
+                }
 
-                kernel_res["status"] = "ok";	
-                kernel_res["payload"] = nl::json::array();	
-                kernel_res["payload"][0] = nl::json::object({	
-                    {"data", {	
-                        {"text/plain", result}	
-                    }},	
-                    {"source", "page"},	
-                    {"start", 0}	
-                });	
-                kernel_res["user_expressions"] = nl::json::object();	
+                kernel_res["status"] = "ok";
+                kernel_res["payload"] = nl::json::array();
+                kernel_res["payload"][0] = nl::json::object({
+                    {"data", {
+                        {"text/plain", result}
+                    }},
+                    {"source", "page"},
+                    {"start", 0}
+                });
+                kernel_res["user_expressions"] = nl::json::object();
 
-                return kernel_res;	
+                return kernel_res;
             }
             code_copy = code;
         }
@@ -171,10 +176,8 @@ namespace xpyt
             register_filename_mapping(filename, execution_count);
 
             // Caching the input code
-#if PY_MAJOR_VERSION >= 3
             py::module linecache = py::module::import("linecache");
             linecache.attr("xupdatecache")(code, filename);
-#endif
 
             // If the last statement is an expression, we compile it separately
             // in an interactive mode (This will trigger the display hook)
@@ -237,6 +240,11 @@ namespace xpyt
             kernel_res["traceback"] = error.m_traceback;
         }
 
+        // Cache inputs
+        py::globals()["_iii"] = py::globals()["_ii"];
+        py::globals()["_ii"] = py::globals()["_i"];
+        py::globals()["_i"] = code;
+
         return kernel_res;
     }
 
@@ -418,4 +426,55 @@ namespace xpyt
         }
         py::globals()["display"] = display_module.attr("display");
     }
+
+    void interpreter::load_extensions()
+    {
+        if (m_has_ipython)
+        {
+            py::module os = py::module::import("os");
+            py::module path = py::module::import("os.path");
+            py::module sys = py::module::import("sys");
+            py::module fnmatch = py::module::import("fnmatch");
+
+            py::str extensions_path = path.attr("join")(sys.attr("exec_prefix"), "etc", "xeus-python", "extensions");
+
+            if (!is_pyobject_true(path.attr("exists")(extensions_path)))
+            {
+                return;
+            }
+
+            py::list list_files = os.attr("listdir")(extensions_path);
+
+            xinteractive_shell* xshell = get_kernel_module()
+                .attr("get_ipython")()
+                .cast<xinteractive_shell*>();
+            py::object extension_manager = xshell->get_extension_manager();
+
+            for (const py::handle& file : list_files)
+            {
+                if (!is_pyobject_true(fnmatch.attr("fnmatch")(file, "*.json")))
+                {
+                    continue;
+                }
+
+                try
+                {
+                    std::ifstream config_file(py::str(path.attr("join")(extensions_path, file)).cast<std::string>());
+                    nl::json config;
+                    config_file >> config;
+
+                    if (config["enabled"].get<bool>())
+                    {
+                        extension_manager.attr("load_extension")(config["module"].get<std::string>());
+                    }
+                }
+                catch (py::error_already_set& e)
+                {
+                    xerror error = extract_error(e);
+
+                    std::cerr << "Warning: Failed loading extension with: " << error.m_ename << ": " << error.m_evalue << std::endl;
+                }
+            }
+        }
+    }
 }
